/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Die moeglichen Varianten fuer den Vorteil Begabung fuer [Talentgruppe]
 * und den Nachteil Unfaehigkeit fuer [Talentgruppe].
 *
 * @author Bernhard Jung
 */
public class VariantenTalentgruppe implements Varianten {

    /** Repraesentiert die 'Talentgruppe' der Nahkampffertigkeiten. */
    public static final String NAHKAMPF =
        I18n.i18n( "VariantenTalentgruppe.Nahkampffertigkeiten" );

    /** Repraesentiert die 'Talentgruppe' der Fernkampffertigkeiten. */
    public static final String FERNKAMPF =
        I18n.i18n( "VariantenTalentgruppe.Fernkampffertigkeiten" );

    /**
     * Die Liste der Talentgruppen und die beiden Strings
     * {@link #NAHKAMPF} und {@link #FERNKAMPF}.
     */
    private final List< Object > varianten;

    VariantenTalentgruppe() {

        varianten = new ArrayList< Object >( 10 );

        varianten.add( Talentgruppe.GESELLSCHAFT );
        varianten.add( Talentgruppe.HANDWERK );
        varianten.add( Talentgruppe.KAMPF );
        varianten.add( NAHKAMPF );
        varianten.add( FERNKAMPF );
        varianten.add( Talentgruppe.KOERPER );
        varianten.add( Talentgruppe.SCHRIFTEN );
        varianten.add( Talentgruppe.SPRACHEN );
        varianten.add( Talentgruppe.NATUR );
        varianten.add( Talentgruppe.WISSEN );

        Collections.sort( varianten, new Comparator<Object>() {

            private final Collator collator = Collator.getInstance();

            public int compare( final Object o1, final Object o2 ) {

                return collator.compare( o1.toString(), o2.toString() );
            }
        });
    }

    /**
     * @throws UnsupportedOperationException wenn die Methode aufgerufen wird.
     */
    public void einfuegen( final String variante ) {
        throw new UnsupportedOperationException();
    }

    /**
     *
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * @return eine Liste der moeglichen Talentgruppen, die Liste enthaelt auch
     *         die beiden Strings {@link #NAHKAMPF} und
     *         {@link #FERNKAMPF}.
     */
    public List<Object> getVarianten(final Held held) {
        return new ArrayList< Object >( varianten );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code false} */
    public boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return "Talentgruppen: " + getVarianten( null ).toString();
    }
}
