/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Repraesentiert einen einzelnen Vor- oder Nachteil, den der Held durch einen
 * Baustein erhalten kann.
 * Speichert evtl, die Id der Variante des Wertes.
 *
 * @author Bernhard Jung
 */
public class VorNachteilWertBonus {

    /**
     * Die Id, die in der XML-Datei fuer die Variante des Wertes verwendet
     * wurde.
     */
    private final String variantenId;

    /** Der Wert den man durch den Bonus erhaelt. */
    private final VorNachteilWert wert;

    /**
     * @param wert den man durch den Bonus erhaelt.
     */
    public VorNachteilWertBonus(
            final VorNachteilWert wert ) {

        this( wert, null );
    }

    /**
     * @param derWert
     *            den man durch den Bonus erhaelt.
     * @param dieVariantenId
     *            Die Id der Variante, die fuer den Wert benutzt wurde. Der Wert
     *            kann {@code null} sein. Es muss sich nicht unbedingt um eine
     *            Id handeln, es kann auch ein Text fuer die Variante sein.
     */
    public VorNachteilWertBonus(
            final VorNachteilWert derWert,
            final String dieVariantenId ) {

        wert = derWert;

        variantenId = dieVariantenId;
    }

    /**
     * @return den Wert den dieser Bonus repraesentiert.
     */
    public final VorNachteilWert getWert() {
        return wert.clone();
    }

    /** @return die Textrepraesentation des Wertes. */
    @Override
    public final String toString() {
        return wert.toString();
    }

    /**
     * @return eine XML-Repraesentation des Bonus.
     */
    public final IXMLElement toXml() {

        final IXMLElement element =
            new XMLElement( TAG_WERT );

        element.setAttribute( ATTR_VORNACHTEIL, wert.getVorNachteil().getId() );

        if ( null != variantenId ) {

            element.setAttribute( ATTR_VARIANTE, variantenId );
        }

        if ( wert.getVorNachteil().hatStufen() ) {

            element.setAttribute( ATTR_STUFE, String.valueOf( wert.getStufe()));
        }

        return element;
    }
}
