/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.voraussetzungen;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

/**
 * Eine Voraussetzung in einer Eigenschaft bei der zB KK &ge; 11 gefordert ist 
 * oder 3 &le; SO &le; 8
 *  
 * @author Bernhard Jung
 */
public abstract class VoraussetzungEigenschaft implements Voraussetzung {

    /** @return die Eigenschaften auf die sich die Voraussetzung bezieht. */
    public abstract Eigenschaft[] getEigenschaften();

    /**
     * @return die Stufe die der Held hoechstens in der Eigenschaft haben darf.
     *      Wenn keine Hoechststufe vorgegeben wird ist der gelieferte Wert
     *      unbestimmt. Diese Implementierung liefert immer 
     *      {@link Integer#MAX_VALUE}.
     */
    public int getHoechststufe() {
        return Integer.MAX_VALUE;
    }
    
    /**
     * @return die Stufe die der Held mindestens in der Eigenschaft haben muss.
     *      Wenn keine Mindeststufe vorgegeben wird ist der gelieferte Wert
     *      unbestimmt. Diese Implementierung liefert immer 0.
     */
    public int getMindeststufe() {
        return 0;
    }
    
    /** @return {@code true} wenn die Voraussetzung fuer den Helden eine
     *      Mindeststufe in der Eigenschaft vorgibt, sonst {@code false}
     *      Diese Implementierung liefert immer {@code false}. 
     */
    public boolean hatHoechststufe() {
        return false;
    }
    
    /**
     * @return {@code true} wenn die Voraussetzung fuer den Helden eine 
     *      Mindeststufe in der Eigenschaft vorgibt, sonst {@code false}.
     *      Diese Implementierung liefert immer {@code true}.
     */
    public boolean hatMindeststufe( final Held held ) {
        return true;
    }
}
