/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.voraussetzungen;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Eine Voraussetzung bei der ein Held ein bestimmtes Geschlecht haben muss.
 *
 * @author Bernhar Jung
 */
public class VoraussetzungGeschlecht implements Voraussetzung {

    /** Das geforderte Geschlecht. */
    private final Geschlecht geschlecht;

    /** Eine kurze Beschreibung der Voraussetzung. */
    public final String beschreibungString;

    /**
     * @param dasGeschlecht das von der Voraussetzung gefordert wird.
     */
    public VoraussetzungGeschlecht( final Geschlecht dasGeschlecht ) {

        geschlecht = dasGeschlecht;

        beschreibungString =  String.format(
                    I18n.i18n( "VoraussetzungGeschlecht" ),
                    geschlecht );
    }

    /**
     * @return das geforderte Geschlecht.
     */
    public final Geschlecht getGeschlecht() {
        return geschlecht;
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen( final Held held ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( geschlecht != held.getGeschlecht() ) {

            beschreibung.einfuegen( beschreibungString );
        }

        return beschreibung;
    }

    /** @return eine kurze Beschreibung der Voraussetzung */
    @Override
    public final String toString() {
        return beschreibungString;
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement element =
            new XMLElement( HeldenbausteinXmlTags.TAG_GESCHLECHT );

        element.setContent( geschlecht.name() );

        return element;
    }
}
