/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factories;

import static de.bernhardjung.dsaprogramm.daten.xml.NahkampfwaffeXmlTags.*;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;

import de.bernhardjung.dsaprogramm.basis.Nahkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class FactoryNahkampfwaffen extends Factory< Nahkampfwaffe > {

    @Override
    public List< Nahkampfwaffe > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< Nahkampfwaffe > waffen = new ArrayList< Nahkampfwaffe >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final IXMLElement waffeElement : wurzelElement.getChildren() ) {

            final String id = waffeElement.getAttribute( ATTR_ID );

            final Nahkampfwaffe waffe = speicher.getNahkampfwaffe( id );

            waffe.setBf( getInt( waffeElement, ATTR_BF ) );

            waffe.setDkH( getBoolean( waffeElement, ATTR_DK_H ) );

            waffe.setDkN( getBoolean( waffeElement, ATTR_DK_N ) );

            waffe.setDkS( getBoolean( waffeElement, ATTR_DK_S ) );

            waffe.setIni( getInt( waffeElement, ATTR_INI ) );

            if ( waffeElement.hasAttribute( ATTR_NAME ) ) {

                waffe.setName( waffeElement.getAttribute( ATTR_NAME ) );
            }

            waffe.setSchadensschritt( getInt( waffeElement,
                                      ATTR_SCHADENSSCHRITT ) );

            waffe.setSchwellenwert( getInt( waffeElement,
                                    ATTR_SCHWELLENWERT ) );

            final String talentId = waffeElement.getAttribute( ATTR_TALENT );

            waffe.setTalent( (Kampftechnik) Talent.getTalent( talentId ) );

            final String tp = waffeElement.getAttribute( ATTR_TP );

            waffe.setTp( new Tp( tp ) );

            waffe.setWmAt( getInt( waffeElement, ATTR_WM_AT ) );

            waffe.setWmPa( getInt( waffeElement, ATTR_WM_PA ) );

            final List< Kampftechnik > talente =
                new ArrayList< Kampftechnik >();

            for ( final IXMLElement talentElement : waffeElement.getChildren()){

                final Kampftechnik talent = (Kampftechnik)
                    Talent.getTalent( talentElement.getContent() );

                talente.add( talent );
            }

            waffe.setTalente( talente );

            waffen.add( waffe );
        }

        return waffen;
    }
}
