/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;

import de.bernhardjung.dsaprogramm.gui.action.ActionBeenden;
import de.bernhardjung.dsaprogramm.gui.action.ActionDatenquelle;
import de.bernhardjung.dsaprogramm.gui.action.ActionFernwaffeneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionHilfe;
import de.bernhardjung.dsaprogramm.gui.action.ActionInfo;
import de.bernhardjung.dsaprogramm.gui.action.ActionKultureneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenHeldXml;
import de.bernhardjung.dsaprogramm.gui.action.ActionNahkampfwaffeneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionNeuerHeld;
import de.bernhardjung.dsaprogramm.gui.action.ActionProfessioneneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionRasseneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionRuestungeneditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionSchildeeditor;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernHeldRtf;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernHeldXml;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaFernkampfwaffen;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaKultur;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaNahkampfwaffen;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaProfession;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaRasse;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaRuestungen;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaSchilde;
import de.bernhardjung.dsaprogramm.gui.steigern.PanelSteigern;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.FileFilter;

public class Hauptpanel extends JPanel {

    /**
     * Die Action um das Programm zu beenden.
     */
    private final Action actionBeenden;

    /** Die Action die den Dialog zu Auswaehlen der Datenquelle aufruft. */
    private final Action actionDatenquelle;

    /** Die Action um den Held aus einer XML-Datei zu laden. */
    private final Action actionLadenXml;

    /** Die Action um den Held als RTF-Datei zu speichern. */
    private final Action actionSpeichernRtf;

    /** Die Action um den Held als XML-Datei zu speichern. */
    private final Action actionSpeichernXml;

    /** Der (Internal)Frame, der das Panel enthaelt. */
    private final Container elternkomponente;

    /** Die JComponent die in der Mitte des Hauptfensters angezeigt wird. */
    private JComponent hauptcomponent;

    /** Das Hauptfenster der Anwendung. */
    private final JFrame hauptfenster;

    /** Das Panel mit dem ein Held erschaffen wird. */
    private PanelErschaffung panelErschaffung;

    /** Das Panel mit dem man die Fernwaffen bearbeiten kann. */
    private PanelMetaFernkampfwaffen panelMetaFernwaffen;

    /** Das Panel mit dem man einen Kultur eingeben kann. */
    private PanelMetaKultur panelMetaKultur;

    /** Das Panel mit dem man die Nahkampfwaffen bearbeiten kann. */
    private PanelMetaNahkampfwaffen panelMetaNahkampfwaffen;

    /** Das Panel mit dem man einen Profession eingeben kann. */
    private PanelMetaProfession panelMetaProfession;

    /** Das Panel mit dem man einen Rasse eingeben kann. */
    private PanelMetaRasse panelMetaRasse;

    /** Der Editor fuer die Ruestungen. */
    private PanelMetaRuestungen panelMetaRuestungen;

    /** Der Editor fuer die Schilde. */
    private PanelMetaSchilde panelMetaSchilde;

    /** Das Panel mit dem ein Held gesteigert wird. */
    private PanelSteigern panelSteigern;

    public Hauptpanel(
            final JFrame dasHauptfenster,
            final Container dieElternkomponente ) {

        super( new BorderLayout() );

        hauptfenster = dasHauptfenster;

        elternkomponente = dieElternkomponente;

        actionBeenden = new ActionBeenden( this );

        actionDatenquelle = new ActionDatenquelle( hauptfenster );

        actionSpeichernRtf = new ActionSpeichernHeldRtf( hauptfenster, this );
        actionSpeichernRtf.setEnabled( false );

        // Zum laden der XML-Dateien der Helden.
        final JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
        fileChooser.setMultiSelectionEnabled( false );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.setFileFilter( new FileFilter( ".dcg" ) );

        actionLadenXml = new ActionLadenHeldXml( this, fileChooser );

        actionSpeichernXml =
            new ActionSpeichernHeldXml( hauptfenster, this, fileChooser );
        actionSpeichernXml.setEnabled( false );

        final JLabel logo = new JLabel(
                Util.ladeIcon( I18n.i18n( "Hauptfenster.Logo" ) ) );
        anzeigen( logo );

        elternkomponente.setSize( 640, 480 );

    }

    /**
     * Maximiert das Hauptfenster so weit, dass sein Inhalt ganz angezeigt wird.
     */
    public final void anpassenGroesse() {

        final Dimension preferred = elternkomponente.getPreferredSize();

        final Dimension size = elternkomponente.getSize();

        if ( preferred.height > size.height ) {
            size.height = preferred.height;
        }

        if ( preferred.width > size.width ) {
            size.width = preferred.width;
        }

        elternkomponente.setSize( size );
    }

    /**
     * Bestimmt was im Hauptfenster angezeigt wird.
     *
     * @param component die dargestellt wird.
     */
    public final void anzeigen( final JComponent component ) {

        if ( null != component
         && component != hauptcomponent ) {

            if ( null != hauptcomponent ) {

                remove( hauptcomponent );

            }

            if ( component.equals( panelSteigern ) ) {

                actionSpeichernXml.setEnabled( true );
                actionSpeichernRtf.setEnabled( true );

            } else {

                actionSpeichernXml.setEnabled( false );
                actionSpeichernRtf.setEnabled( false );

            }

            hauptcomponent = component;

            add( hauptcomponent, BorderLayout.CENTER );

            anpassenGroesse();

            validate();
            repaint();
        }
    }

    /**
     * @param aktionBeendenErzeugen
     *            {@code true}, wenn das Programm als alleinstehende Anwendung
     *            laeuft und es einen Menueeintrag geben soll, der die JVM
     *            beendet. {@code false}, wenn das Programm als Plugin im
     *            DSA Desktop laeuft.
     *
     * @return das Menue der Anwendung.
     */
    public final JMenuBar erzeugeMenue( final boolean aktionBeendenErzeugen ) {

        final JMenuBar menue = new JMenuBar();

        final JMenu aktionen = new JMenu(
                I18n.i18n( "Hauptfenster.Menue.Aktionen" ) );
        String mnemonic =
            I18n.i18n( "Hauptfenster.Menue.Aktionen.Mnemonic" );
        aktionen.setMnemonic( Util.getKeyCode( mnemonic ) );

        aktionen.add( new ActionHilfe() );

        aktionen.add( new ActionInfo( hauptfenster ) );

        aktionen.addSeparator();

        aktionen.add( new ActionNeuerHeld( this ) );

        aktionen.add( actionSpeichernRtf );

        aktionen.add( actionLadenXml );

        aktionen.add( actionSpeichernXml );

        if ( aktionBeendenErzeugen ) {

            aktionen.addSeparator();

            aktionen.add( actionBeenden );
        }

        menue.add( aktionen );

        final JMenu daten = new JMenu(
                I18n.i18n( "Hauptfenster.Menue.Daten" ) );
        mnemonic = I18n.i18n( "Hauptfenster.Menue.Daten.Mnemonic" );
        daten.setMnemonic( Util.getKeyCode( mnemonic ) );

        daten.add( actionDatenquelle );

        daten.add( new ActionRasseneditor( this ) );

        daten.add( new ActionKultureneditor( this ) );

        daten.add( new ActionProfessioneneditor( this ) );

        daten.add( new ActionRuestungeneditor( this ) );

        daten.add( new ActionNahkampfwaffeneditor( this ) );

        daten.add( new ActionFernwaffeneditor( this ) );

        daten.add( new ActionSchildeeditor( this ) );

        menue.add( daten );

        return menue;
    }

    /**
     * @return die Action um das Programm zu beenden, wenn es als alleinstehende
     *         Anwendung laeuft.
     */
    public final Action getActionBeenden() {
        return actionBeenden;
    }

    /**
     * @return das Panel, das zum Erschaffen von Helden verwendet wird.
     */
    public final PanelErschaffung getPanelErschaffung() {
        if ( null == panelErschaffung ) {
            panelErschaffung = new PanelErschaffung();
        }
        return panelErschaffung;
    }

    /***
     * @return das Panel  mit dem man die Fernwaffen bearbeiten kann.
     */
    public final PanelMetaFernkampfwaffen getPanelMetaFernwaffen() {
        if ( null == panelMetaFernwaffen ) {
            panelMetaFernwaffen = new PanelMetaFernkampfwaffen();
        }
        return panelMetaFernwaffen;
    }

    /**
     * @return das Panel, mit dem man eine Kultur eingeben kann.
     */
    public final PanelMetaKultur getPanelMetaKultur() {
        if ( null == panelMetaKultur ) {
            panelMetaKultur = new PanelMetaKultur( hauptfenster, this );
        }
        return panelMetaKultur;
    }

    /***
     * @return das Panel  mit dem man die Nahkampfwaffen bearbeiten kann.
     */
    public final PanelMetaNahkampfwaffen getPanelMetaNahkampfwaffen() {
        if ( null == panelMetaNahkampfwaffen ) {
            panelMetaNahkampfwaffen = new PanelMetaNahkampfwaffen();
        }
        return panelMetaNahkampfwaffen;
    }

    /**
     * @return das Panel, mit dem man eine Profession eingeben kann.
     */
    public final PanelMetaProfession getPanelMetaProfession() {
        if ( null == panelMetaProfession ) {
            panelMetaProfession = new PanelMetaProfession( hauptfenster, this );
        }
        return panelMetaProfession;
    }

    /**
     * @return das Panel, mit dem man eine Rasse eingeben kann.
     */
    public final PanelMetaRasse getPanelMetaRasse() {
        if ( null == panelMetaRasse ) {
            panelMetaRasse = new PanelMetaRasse( hauptfenster, this );
        }
        return panelMetaRasse;
    }

    /**
     * @return das Panel, mit dem man die Ruestungen editieren kann.
     */
    public final PanelMetaRuestungen getPanelMetaRuestungen() {
        if ( null == panelMetaRuestungen ) {
            panelMetaRuestungen = new PanelMetaRuestungen();
        }
        return panelMetaRuestungen;
    }

    /**
     * @return den Editor fuer die Schilde.
     */
    public final PanelMetaSchilde getPanelMetaSchilde() {
        if ( null == panelMetaSchilde ) {
            panelMetaSchilde = new PanelMetaSchilde();
        }
        return panelMetaSchilde;
    }

    /**
     * @return das Panel das zum Steigern von Helden verwendet wird.
     */
    public final PanelSteigern getPanelSteigern() {
        if ( null == panelSteigern ) {
            panelSteigern = new PanelSteigern();
        }
        return panelSteigern;
    }

    /**
     * Ruft den Dialog zu Auswaehlen der Datenquelle auf.
     */
    public void zeigeDialogDatenquelle() {
        actionDatenquelle.actionPerformed( null );
    }
}
