/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;
import java.net.URL;

import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.swing.AbstractAction;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Ruft den Hilfe-Dialog auf.
 * 
 * @author Bernhard Jung
 */
public class ActionHilfe extends AbstractAction {

    /** Der Pfad zum Standard Helpset. */
    private static final String HELPSET = 
        "de/bernhardjung/dsaprogramm/hilfe/hilfe.hs";

    /**
     * Die Taste mit der die Aktion im Menue direkt angesprungen werden kann.
     */
    private static final String MNEMONIC = 
        I18n.i18n( "ActionHilfe.Mnemonic" );
        
    /** Das Tastenkuerzel mit dem die Aktion ausgeloest werden kann. */
    private static final String TASTEN = I18n.i18n( "ActionHilfe.Tasten" );
    
    /** Repraesentiert den Hilfe-Dialog. */
    private HelpBroker helpBroker; 
    
    public ActionHilfe() {
        
        super( I18n.i18n( "ActionHilfe.Name" ),
                Util.ladeIcon( "hilfe" ) );
        
        putValue( MNEMONIC_KEY, Util.getKeyCode( MNEMONIC ) );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( TASTEN ) );
    }
    
    /**
     * Zeigt/Verbirgt den Hilfe Dialog.
     * 
     * @param e ungenutzt.
     */
    public void actionPerformed( final ActionEvent e) {
        
        if ( null == helpBroker ) {
            
            HelpSet hs = null;
            
            try {
                
                final ClassLoader cl = ActionHilfe.class.getClassLoader();
                
                final URL hsURL = HelpSet.findHelpSet( cl, HELPSET );
                
                hs = new HelpSet( cl, hsURL );
             
                helpBroker = hs.createHelpBroker();
                
            } catch (Exception ex) {
                
                DialogFehlermeldung.zeigeDialog( ex );
                
                return;
            }            
        }
        
        helpBroker.setDisplayed( ! helpBroker.isDisplayed() );
        
    }
}
