/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.erschaffung.Erschaffung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Die Aktion die eine neue Heldenerschaffung beginnt.
 *
 * @author Bernhard Jung
 */
public class ActionNeuerHeld extends AbstractAction {

    /** Das Tastenkurzel mit dem die Aktion ausgeloest werden kann. */
    private static final String TASTEN = I18n.i18n( "ActionNeuerHeld.Tasten" );

    /**
     * Die Taste mit der die Aktion im Menue direkt angesprungen werden kann.
     */
    private static final String MNEMONIC =
        I18n.i18n( "ActionNeuerHeld.Mnemonic" );

    /** Das Objekt das die Erschaffung eines neuen Helden steuert. */
    private final Erschaffung erschaffung;

    public ActionNeuerHeld( final Hauptpanel hauptpanel ) {

        super( I18n.i18n( "ActionNeuerHeld.Name" ),
                Util.ladeIcon( "neuerHeld" ) );

        erschaffung = new Erschaffung( hauptpanel );

        putValue( MNEMONIC_KEY, Util.getKeyCode( MNEMONIC ) );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( TASTEN ) );
    }

    /**
     * Startet eine neue Heldenerschaffung
     *
     * @param e ungenutzt.
     */
    public void actionPerformed( final ActionEvent e ) {
        erschaffung.starten();
    }

}
