/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import de.bernhardjung.dsaprogramm.zauber.Zaubergruppe;

/**
 * Erlaubt es die zusaetzlichen Leitzauber fuer elfische Helden zu waehlen.
 * Es wird einfach eine Liste von Checkboxen angezeigt, die man anwaehlen kann.
 *
 * @author Bernhard Jung
 *
 */
public class PanelLeitzauber extends JPanel implements ActionListener {

    /** Erklaert dem Benutzer wie viele Leitzauber gewaehlt werden sollen. */
    private static final String HINWEIS =
        I18n.i18n( "PanelLeitzauber.Hinweis" );

    /** Die Anzahl Leitzauber die gewaehlt werden soll. */
    private int anzahl;

    /** Die Anzahl der Zauber die aktuell ausgewaehlt sind. */
    private int anzahlGewaehlte;

    private final List< JCheckBox > checkboxen;

    /** Die Zauber die zur Auswahl stehen. */
    private final List< Zauber> zauber;

    public PanelLeitzauber() {

        super( new BorderLayout() );

        anzahlGewaehlte = 0;

        checkboxen = new ArrayList< JCheckBox >();

        zauber = new ArrayList< Zauber >();
    }

    /**
     * Deaktivert die JCheckBoxen, wenn alle Leitzauber gewaehlt wurden.
     */
    public void actionPerformed( final ActionEvent e ) {

        final JCheckBox checkbox = (JCheckBox) e.getSource();

        if ( checkbox.isSelected() ) {

            ++anzahlGewaehlte;

        } else {

            --anzahlGewaehlte;
        }

        if ( anzahlGewaehlte == anzahl ) {

            for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

                final JCheckBox c = checkboxen.get( i );

                c.setEnabled( c.isSelected() );
            }

        } else {

            for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

                checkboxen.get( i ).setEnabled( true );

            }

        }
    }

    /** Baut die Benutzeroberflaeche neu auf. */
    private final void aufbauenBenutzeroberflaeche() {

        removeAll();

        initCheckBoxen();

        // Das Panel auf das die ganzen Elemente kommen.
        final JPanel panel = new JPanel();

        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        final String hinweis = String.format( HINWEIS, anzahl );

        panel.add( new JLabel( hinweis ), cc.xyw( 2, 2, 3 ) );

        int y = 2;

        for ( int i = 0; i < zauber.size() ; ++i ) {

            y += 2;

            panel.add( checkboxen.get( i ), cc.xy( 3, y ) );
        }

        final JScrollPane scrollPane = new JScrollPane( panel );
        // HACK: Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( this.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        scrollPane.getVerticalScrollBar()
                .setUnitIncrement( Util.UNIT_INCREMENT );

        add( scrollPane );
    }

    /**
     * Erzeugt ein Layout um fuer jeden Zauber eine Checkbox darauf zu plazieren.
     * @return
     */
    private final FormLayout erzeugeLayout() {

        final StringBuilder zeilen = new StringBuilder();

        zeilen.append( "7dlu, P, " );

        for ( int i = zauber.size() ; i > 0 ; --i ) {

            zeilen.append( "4dlu, P, " );
        }

        zeilen.append( "7dlu" );

        final String spalten = "7dlu, P:G, P, P:G, 7dlu";

        return new FormLayout( spalten, zeilen.toString() );
    }

    /**
     * @return die ausgewaehlten Leitzauber.
     */
    public final List< Zauber > getGewaehlteZauber() {

        final List< Zauber > gewaehlte = new ArrayList< Zauber >( anzahl );

        for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

            if ( checkboxen.get( i ).isSelected() ) {

                gewaehlte.add( zauber.get( i ) );
            }
        }

        return gewaehlte;
    }

    /**
     * Sort dafuer das fuer jeden Zauber eine Checkbox existiert und
     * setzt den Text der Checkboxen auf die Namen der Zauber.
     */
    private final void initCheckBoxen() {

        for ( int i = checkboxen.size(), n = zauber.size(); i < n ; ++i ) {

            final JCheckBox checkbox = new JCheckBox();

            checkbox.addActionListener( this );

            checkboxen.add(  checkbox );
        }

        for ( int i = 0, n = zauber.size() ; i < n ; ++i ) {

            final JCheckBox checkbox = checkboxen.get( i );

            checkbox.setText( zauber.get( i ).getName() );

            checkbox.setSelected( false );
        }
    }

    /**
     * @param held fuer den die Leitzauber gewaehlt werden.
     * @param anzahlLeitzauber die gewaehlt werden soll.
     */
    public final void setDaten( final Held held, final int anzahlLeitzauber ) {

        anzahl = anzahlLeitzauber;

        anzahlGewaehlte = 0;

        zauber.clear();

        for ( final Zauber z : Zaubergruppe.ZAUBER ) {

            final ZauberWert wert;

            if ( z.hatVarianten() ) {

                wert = new ZauberWert( z, "", 0 );

            } else {

                wert = new ZauberWert( z );

            }

            final KonfliktBeschreibung beschreibung = wert.pruefen( held );

            if ( held.hatLeitzauber( z )
              || beschreibung.enthaeltKonflikt() ) {

                continue;
            }

            zauber.add( z );
        }

        aufbauenBenutzeroberflaeche();

        validate();
        repaint();
    }
}
