/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenKultur;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernKultur;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Das Panel zum Eingeben einer Kultur.
 *
 * @author Bernhard Jung
 *
 */
public class PanelMetaKultur extends JPanel {

    private static final String DIVERSES_FRAME =
        I18n.i18n( "PanelMetaKultur.Diverses" );

    private static final String DIVERSES_REITER =
        I18n.i18n( "PanelMetaKultur.Diverses.Reiter" );

    private static final String EMPF_VOR_NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaKultur.EmpfohleneVorNachteile" );

    private static final String EMPF_VOR_NACHTEILE_REITER =
        I18n.i18n( "PanelMetaKultur.EmpfohleneVorNachteile.Reiter" );

    private static final String LEITTALENTE_FRAME =
        I18n.i18n( "PanelMetaKultur.Leittalente" );

    private static final String LEITTALENTE_REITER =
        I18n.i18n( "PanelMetaKultur.Leittalente.Reiter" );

    private static final String MODIFIKATIONEN_FRAME =
        I18n.i18n( "PanelMetaKultur.Modifikationen" );

    private static final String MODIFIKATIONEN_REITER =
        I18n.i18n( "PanelMetaKultur.Modifikationen.Reiter" );

    private static final String NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaKultur.Nachteile" );

    private static final String NACHTEILE_REITER =
        I18n.i18n( "PanelMetaKultur.Nachteile.Reiter" );

    private static final String NEU =
        I18n.i18n( "PanelMetaKultur.Neu" );

    private static final String PROFESSIONEN_FRAME =
        I18n.i18n( "PanelMetaKultur.Professionen" );

    private static final String PROFESSIONEN_REITER =
        I18n.i18n( "PanelMetaKultur.Professionen.Reiter" );

    private static final String SONDERFERTIGKEITEN_FRAME =
        I18n.i18n( "PanelMetaKultur.Sonderfertigkeiten" );

    private static final String SONDERFERTIGKEITEN_REITER =
        I18n.i18n( "PanelMetaKultur.Sonderfertigkeiten.Reiter" );

    private static final String TALENTE_FRAME =
        I18n.i18n( "PanelMetaKultur.Talente" );

    private static final String TALENTE_REITER =
        I18n.i18n( "PanelMetaKultur.Talente.Reiter" );

    private static final String UNGE_VOR_NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaKultur.UngeeigneteVorNachteile" );

    private static final String UNGE_VOR_NACHTEILE_REITER =
        I18n.i18n( "PanelMetaKultur.UngeeigneteVorNachteile.Reiter" );

    private static final String VERBILLIGTE_SF_FRAME =
        I18n.i18n( "PanelMetaKultur.VerbilligteSonderfertigkeiten" );

    private static final String VERBILLIGTE_SF_REITER =
        I18n.i18n( "PanelMetaKultur.VerbilligteSonderfertigkeiten.Reiter" );

    private static final String VERBILLIGTE_VARIANTEN_FRAME =
        I18n.i18n( "PanelMetaKultur.VerbilligteVarianten" );

    private static final String VERBILLIGTE_VARIANTEN_REITER =
        I18n.i18n( "PanelMetaKultur.VerbilligteVarianten.Reiter" );

    private static final String VORTEILE_FRAME =
        I18n.i18n( "PanelMetaKultur.Vorteile" );

    private static final String VORTEILE_REITER =
        I18n.i18n( "PanelMetaKultur.Vorteile.Reiter" );

    private static final String ZAUBER_FRAME =
        I18n.i18n( "PanelMetaKultur.Zauber" );

    private static final String ZAUBER_REITER =
        I18n.i18n( "PanelMetaKultur.Zauber.Reiter" );

    private static final String ZWEITSPRACHE_FRAME =
        I18n.i18n( "PanelMetaKultur.Zweitsprache" );

    private static final String ZWEITSPRACHE_REITER =
        I18n.i18n( "PanelMetaKultur.Zweitsprache.Reiter" );

    private final PanelMetaKulturDiverses diverses;

    private final PanelMetaListe< VorNachteil > empfohlene;

    private final PanelMetaLeittalente leittalente;

    private final PanelMetaModifikationen modifikationen;

    private final PanelMetaVorNachteile nachteile;

    private final PanelMetaMoeglicheProfessionen professionen;

    private final PanelMetaSonderfertigkeiten sonderfertigkeiten;

    private final PanelMetaTalentboni talentboni;

    private final PanelMetaUngeeigneteVorNachteile ungeeignete;

    private final PanelMetaVerbilligteSonderfertigkeiten verbilligteSf;

    private final PanelMetaVerbilligteVarianten verbilligteVarianten;

    private final PanelMetaVorNachteile vorteile;

    private final PanelMetaZauberboni zauberboni;

    private final PanelMetaListe< Sprache > zweitsprache;

    @SuppressWarnings( "unchecked")
    public PanelMetaKultur(
            final JFrame hauptfenster,
            final Hauptpanel hauptpanel ) {

        diverses = new PanelMetaKulturDiverses( this );

        empfohlene = new PanelMetaListe< VorNachteil>(
                VorNachteil.getVorNachteile() );

        leittalente = new PanelMetaLeittalente();

        modifikationen = new PanelMetaModifikationen(
                Collections.EMPTY_LIST );

        nachteile = new PanelMetaVorNachteile( VorNachteil.getNachteile(),
                                               NACHTEILE_REITER );

        professionen = new PanelMetaMoeglicheProfessionen();

        sonderfertigkeiten = new PanelMetaSonderfertigkeiten();

        talentboni = new PanelMetaTalentboni(
                PanelMetaTalentboni.Modus.KULTUR );

        ungeeignete = new PanelMetaUngeeigneteVorNachteile();

        verbilligteSf = new PanelMetaVerbilligteSonderfertigkeiten();

        verbilligteVarianten = new PanelMetaVerbilligteVarianten();

        vorteile = new PanelMetaVorNachteile( VorNachteil.getVorteile(),
                                              VORTEILE_REITER );

        zauberboni = new PanelMetaZauberboni();

        final List< Sprache > sprachen =
            new ArrayList< Sprache >( Talentgruppe.SPRACHEN.size() );

        for ( final Talent sprache : Talentgruppe.SPRACHEN ) {
            sprachen.add( (Sprache) sprache );
        }

        zweitsprache = new PanelMetaListe< Sprache >( sprachen );

        // Button Bar
        // **********
        final JButton neu = new JButton( NEU );
        neu.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                neu();
            }
        });

        final JPanel buttonBar = new JPanel(
                new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );

        final CellConstraints cc = new CellConstraints();

        buttonBar.add( ButtonBarFactory.buildLeftAlignedBar(
                    new JButton( new ActionLadenKultur( this ) ),
                    new JButton( new ActionSpeichernKultur(
                                        hauptfenster,
                                        hauptpanel ) ),
                    neu ),
                    cc.xy( 2, 2 ) );

        // Tabbed Pane
        // ***********
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        tabbedPane.addTab( DIVERSES_REITER,
                           erzeugeFrame( DIVERSES_FRAME, diverses ) );

        tabbedPane.addTab( MODIFIKATIONEN_REITER,
                           erzeugeFrame( MODIFIKATIONEN_FRAME,
                                         Util.erzeugeScrollPane( modifikationen ) ) );

        tabbedPane.addTab( PROFESSIONEN_REITER,
                           erzeugeFrame( PROFESSIONEN_FRAME, professionen ) );

        tabbedPane.addTab( ZWEITSPRACHE_REITER,
                           erzeugeFrame( ZWEITSPRACHE_FRAME,
                                         Util.erzeugeScrollPane( zweitsprache ) ) );

        tabbedPane.addTab( TALENTE_REITER,
                           erzeugeFrame( TALENTE_FRAME, talentboni ) );

        tabbedPane.addTab( LEITTALENTE_REITER,
                           erzeugeFrame( LEITTALENTE_FRAME,
                                         Util.erzeugeScrollPane( leittalente ) ) );

        tabbedPane.addTab( VORTEILE_REITER,
                           erzeugeFrame( VORTEILE_FRAME, vorteile ) );

        tabbedPane.addTab( NACHTEILE_REITER,
                           erzeugeFrame( NACHTEILE_FRAME, nachteile ) );

        tabbedPane.addTab( EMPF_VOR_NACHTEILE_REITER,
                           erzeugeFrame( EMPF_VOR_NACHTEILE_FRAME,
                                         Util.erzeugeScrollPane( empfohlene ) ) );

        tabbedPane.addTab( UNGE_VOR_NACHTEILE_REITER,
                           erzeugeFrame( UNGE_VOR_NACHTEILE_FRAME,
                                         Util.erzeugeScrollPane( ungeeignete ) ) );

        tabbedPane.addTab( SONDERFERTIGKEITEN_REITER,
                           erzeugeFrame( SONDERFERTIGKEITEN_FRAME,
                                         sonderfertigkeiten ) );

        tabbedPane.addTab( VERBILLIGTE_SF_REITER,
                           erzeugeFrame( VERBILLIGTE_SF_FRAME, verbilligteSf ) );

        tabbedPane.addTab( VERBILLIGTE_VARIANTEN_REITER,
                erzeugeFrame( VERBILLIGTE_VARIANTEN_FRAME,
                              verbilligteVarianten ) );

        tabbedPane.addTab( ZAUBER_REITER,
                           erzeugeFrame( ZAUBER_FRAME, zauberboni ) );

        setLayout( new BorderLayout() );

        add( buttonBar, BorderLayout.PAGE_START  );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Kultur kultur ) {

        diverses.anzeigen( kultur );

        empfohlene.setGewaehlteElemente( kultur.getEmpfohleneVorNachteile() );

        professionen.anzeigen( kultur );

        leittalente.anzeigen( kultur );

        modifikationen.anzeigen( kultur );

        nachteile.anzeigen( kultur.getNachteile() );

        sonderfertigkeiten.anzeigen( kultur.getSonderfertigkeiten() );

        talentboni.anzeigen( kultur );

        ungeeignete.anzeigen( kultur.getUngeeigneteVorNachteile() );

        verbilligteSf.anzeigen( kultur );

        verbilligteVarianten.anzeigen( kultur );

        vorteile.anzeigen( kultur.getVorteile() );

        zauberboni.anzeigen( kultur );

        zweitsprache.setGewaehlteElemente( kultur.getZweitsprachen() );
    }

    private SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JComponent inhalt ) {

        final SimpleInternalFrame frame =
            new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( inhalt );

        return frame;
    }

    /** @return die fuer die Kultur gewaehlte Id. */
    public String getKulturId() {
        return diverses.getKulturId();
    }

    /** @return den Namen, den der Benutzer fuer die Rasse angegeben hat. */
    public String getKulturName() {
        return diverses.getKulturName();
    }

    /**
     * Setzt das Panel wieder zurueck, damit man eine neue Kultur
     * eingeben kann.
     */
    private void neu() {

        diverses.reset();

        empfohlene.reset();

        professionen.reset();

        leittalente.reset();

        modifikationen.reset();

        nachteile.reset();

        sonderfertigkeiten.reset();

        talentboni.reset();

        ungeeignete.reset();

        verbilligteSf.reset();

        verbilligteVarianten.reset();

        vorteile.reset();

        zauberboni.reset();

        zweitsprache.reset();
    }

    public final void uebertrageDaten( final Kultur kultur ) {

        diverses.uebertrageDaten( kultur );

        kultur.setEmpfohlene( empfohlene.getGewaehlteElemente() );

        kultur.setProfessionen( professionen.getMoeglicheProfessionen() );

        leittalente.uebertrageDaten( kultur );

        modifikationen.uebertrageDaten( kultur );

        kultur.setNachteile( nachteile.getBoni() );

        kultur.setSonderfertigkeiten( sonderfertigkeiten.getBoni() );

        kultur.setTalente( talentboni.getBoni() );

        kultur.setUngeeignete( ungeeignete.getGewaehlteVorNachteile() );

        kultur.setVerbilligte(
                verbilligteSf.getVerbilligteSonderfertigkeiten() );

        kultur.setVerbilligteVariante(
                verbilligteVarianten.getVerbilligteVarianten() );

        kultur.setVorteile( vorteile.getBoni() );

        kultur.setZauberboni( zauberboni.getBoni() );

        kultur.setZweitsprachen( zweitsprache.getGewaehlteElemente() );
    }

}
