/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.kosten.Kosten;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenRasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.VariantenListModel;
import de.bernhardjung.dsaprogramm.i18n.I18n;

class PanelMetaProfessionDiverses
extends JPanel
implements Observer {

    /**
     * Der Text fuer den Knopf zum Berechnen der GP-Kosten.
     */
    private static final String BERECHNEN =
        I18n.i18n( "PanelMetaProfessionDiverses.Berechnen" );

    /** Der Text neben dem Textfeld fuer die Id. */
    private static final String ID =
        I18n.i18n( "PanelMetaProfessionDiverses.Id" );

    /**
     * Der Text der Checkbox, mit der man angibt ob es sich um eine
     * Kampferprofession handelt.
     */
    private static final String KAEMPFERPROFESSION =
        I18n.i18n( "PanelMetaProfessionDiverses.Kaempferprofession" );

    /**
     * Der Text neben dem Spinner, mit dem die Kosten eingegeben werden koennen.
     */
    private static final String KOSTEN =
        I18n.i18n( "PanelMetaProfessionDiverses.Kosten" );

    /**
     * Der Text neben dem Textfeld, in das man den maennlichen Namen der
     * Profession eingibt.
     */
    private static final String NAME_MAENNLICH =
        I18n.i18n( "PanelMetaProfessionDiverses.Name.Maennlich" );

    /**
     * Der Text neben dem Textfeld, in das man den weiblichen Namen der
     * Profession eingibt.
     */
    private static final String NAME_WEIBLICH =
        I18n.i18n( "PanelMetaProfessionDiverses.Name.Weiblich" );

    /**
     * Der Text auf der Checkbox, mit der man angeben kann, ob die Kosten
     * fuer eine bestimmte Rasse anders sein sollen.
     */
    private static final String RASSE =
        I18n.i18n( "PanelMetaProfessionDiverses.Rasse" );

    /**
     * Der Text neben dem Spinner in dem man die Punkte auf Verbindungen
     * einstellt.
     */
    private static final String VERBINDUNGEN =
        I18n.i18n( "PanelMetaProfessionDiverses.Verbindungen" );

    /**
     * Der Text der Checkbox, mit der man angibt, ob es sich um eine
     * zeitaufwaendige Profession handelt.
     */
    private static final String ZEITAUFWAENDIG =
        I18n.i18n( "PanelMetaProfessionDiverses.Zeitaufwaendig" );


    /** Das Textfeld fuer die Id. */
    private final JTextField id;

    /**
     * Die Checkbox um anzugeben ob es sich um eine Kaempferprofession handelt.
     */
    private final JCheckBox kaempferprofession;

    /** Der Spinner zum einstellen der Kosten. */
    private final JSpinner kosten;

    /**
     * Das Label mit dem die vom Programm berechneten Kosten angezeigt werden.
     */
    private final JLabel kostenLabel;

    /** Das Textfeld, um den Namen fuer Maenner einzugeben. */
    private final JTextField nameMaennlich;

    /** Das Textfeld, um den Namen fuer Frauen einzugeben. */
    private final JTextField nameWeiblich;

    /**
     * Die Checkbox mit der angegeben werden kann, dass eine bestimmte Rasse
     * andere Kosten fuer die Profession zahlen muss.
     */
    private final JCheckBox rasseCheckBox;

    /**
     * Die Combobox mit der die Rasse gewaehlt werden kann, die andere Kosten zu
     * zahlen hat.
     */
    private final JComboBox rasseComboBox;

    /** Das Model fuer die Combobox mit den Rassen. */
    private final VariantenListModel rasseModel;

    /** Die Kosten, die die spezielle Rasse zahlen muss. */
    private final JSpinner rasseSpinner;

    /** Der Spinner mit dem sich die Punkte fuer Verbindungen angeben lassen. */
    private final JSpinner verbindungen;

    /**
     * Die Checkbox mit der sich angeben laesst, dass die Profession
     * zeitaufwaendig ist.
     */
    private final JCheckBox zeitaufwaendig;

    PanelMetaProfessionDiverses( final PanelMetaProfession panelProfession ) {

        id = new JTextField();

        kaempferprofession = new JCheckBox( KAEMPFERPROFESSION );

        kosten = Util.erzeugeSpinner( 0, -99, 99, 1 );

        kostenLabel = new JLabel();

        nameMaennlich = new JTextField();

        nameWeiblich = new JTextField();

        rasseCheckBox = new JCheckBox( RASSE );

        final Datenspeicher speicher = Datenspeicher.getInstanz();
        speicher.addObserver( this );

        rasseModel = new VariantenListModel();

        rasseModel.setItems( new ArrayList< Object >( speicher.getRassen() ) );

        rasseComboBox = new JComboBox( rasseModel );

        rasseSpinner = Util.erzeugeSpinner( 0, 0, 99, 1 );

        verbindungen = Util.erzeugeSpinner( 0, 0, 99, 1 );

        zeitaufwaendig = new JCheckBox( ZEITAUFWAENDIG );

        final JButton berechnen = new JButton( BERECHNEN );
        berechnen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {

                final Profession profession = new Profession( "Profession" );

                panelProfession.uebertrageDaten( profession );

                final double kosten = profession.berechneGpKosten();

                kostenLabel.setText( String.format( "%1$.1f", kosten ) );
            }
        });

        final FormLayout layout =
            new FormLayout( "P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P:G" );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        builder.append( NAME_MAENNLICH, nameMaennlich, 7 );

        builder.append( NAME_WEIBLICH, nameWeiblich, 7 );

        builder.append( ID, id, 7 );

        builder.append( KOSTEN, kosten );
        builder.append( berechnen, kostenLabel );

        builder.nextLine();

        builder.append( "", rasseCheckBox );
        builder.append( rasseComboBox, rasseSpinner );

        builder.nextLine();

        builder.append( VERBINDUNGEN, verbindungen );

        builder.nextLine();

        builder.append( kaempferprofession, 9 );

        builder.append( zeitaufwaendig, 9 );
    }

    public final void anzeigen( final Profession profession ) {

        anzeigenId( profession.getId() );

        kaempferprofession.setSelected( profession.istKaempferprofession() );

        anzeigenKosten( profession.getKostenObjekt() );

        nameMaennlich.setText( profession.getName() );

        nameWeiblich.setText( profession.getNameWeiblich() );

        verbindungen.setValue( profession.getVerbindungen() );

        zeitaufwaendig.setSelected( profession.istZeitaufwaendig() );
    }

    private void anzeigenId( final String idString ) {

        final int index = idString.indexOf( "." );

        final String suffix = idString.substring( 1 + index );

        id.setText( suffix );
    }
    private final void anzeigenKosten( final Kosten k ) {

        if ( k instanceof KostenRasse ) {

            final KostenRasse kostenRasse = (KostenRasse) k;

            kosten.setValue( kostenRasse.getGpKostenNormal() );

            rasseCheckBox.setSelected( true );

            rasseComboBox.setSelectedItem( kostenRasse.getRasse() );

            rasseSpinner.setValue( kostenRasse.getGpKostenRasse() );

        } else {

            kosten.setValue( k.getGpKosten( null ) );

            rasseCheckBox.setSelected( false );

        }
    }

    /** @return die fuer die Profession gewaehlte Id. */
    public String getProfessionId() {
        return "Profession." + id.getText().trim();
    }

    /**
     * @return die maennliche Bezeichnung fuer die Profession.
     */
    public String getProfessionName() {
        return nameMaennlich.getText();
    }

    /** Loescht die Eintraege im Panel. */
    public void reset() {

        id.setText( "" );

        kaempferprofession.setSelected( false );

        kostenLabel.setText( "" );

        nameMaennlich.setText( "" );

        nameWeiblich.setText( "" );

        verbindungen.setValue( 0 );

        zeitaufwaendig.setSelected( false );
    }

    public final void uebertrageDaten( final Profession profession ) {

        final Kosten kostenObjekt;

        if ( rasseCheckBox.isSelected() ) {

            kostenObjekt = new KostenRasse(
                    (Integer) kosten.getValue(),
                    (Rasse) rasseComboBox.getSelectedItem(),
                    (Integer) rasseSpinner.getValue() );

        } else {

            kostenObjekt = new KostenFest( (Integer) kosten.getValue() );
        }

        profession.setKosten( kostenObjekt );

        profession.setKaempferprofession( kaempferprofession.isSelected() );

        profession.setName( nameMaennlich.getText() );

        profession.setNameWeiblich( nameWeiblich.getText() );

        profession.setVerbindungen( (Integer) verbindungen.getValue() );

        profession.setZeitaufwaendig( zeitaufwaendig.isSelected() );
    }

    public void update( final Observable o, final Object arg ) {

        if ( Datenspeicher.RASSEN.equals( arg ) ) {

            final Datenspeicher speicher = Datenspeicher.getInstanz();

            rasseModel.setItems(
                    new ArrayList< Object >( speicher.getRassen() ) );
        }
    }
}
