/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteVariante;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;

class PanelMetaVerbilligteVarianten extends JPanel {

    /**
     * Der Text auf dem Reiter fuer das Panel mit den Sonderfertigkeiten
     * Kulturkunde, Ortskenntnis und Ruestungsgewoehnung.
     */
    private static final String VERSCHIEDENE =
        I18n.i18n( "PanelMetaVerbilligteVarianten.Verschiedene" );

    private final PanelMetaVerbilligteVariantenListe verschiedene;

    private final PanelMetaVerbilligteVariantenListe talentspezialisierungen;

    private final PanelMetaVerbilligteVariantenListe waffenspezialisierungen;

    PanelMetaVerbilligteVarianten() {

        final List< Sonderfertigkeit > verschiedeneSf = Arrays.asList(
                Sonderfertigkeit.KULTURKUNDE,
                Sonderfertigkeit.ORTSKENNTNIS,
                Sonderfertigkeit.RUESTUNGSGEWOEHNUNG_I );

        verschiedene = new PanelMetaVerbilligteVariantenListe( verschiedeneSf );

        talentspezialisierungen = new PanelMetaVerbilligteVariantenListe(
                Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG );

        waffenspezialisierungen = new PanelMetaVerbilligteVariantenListe(
                Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG );

        final JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab( VERSCHIEDENE,
                           Util.erzeugeScrollPane( verschiedene ) );

        tabbedPane.addTab(
                Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG.getKuerzel(),
                Util.erzeugeScrollPane( talentspezialisierungen ) );

        tabbedPane.addTab(
                Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG.getKuerzel(),
                Util.erzeugeScrollPane( waffenspezialisierungen ) );

        setLayout( new BorderLayout() );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Heldenbaustein baustein ) {

        verschiedene.anzeigen( baustein );

        talentspezialisierungen.anzeigen( baustein );

        waffenspezialisierungen.anzeigen( baustein );
    }

    public final List< VerbilligteVariante > getVerbilligteVarianten() {

        final List< VerbilligteVariante > verbilligte =
            new ArrayList< VerbilligteVariante >();

        verbilligte.addAll( verschiedene.getVerbilligteVarianten() );

        verbilligte.addAll( talentspezialisierungen.getVerbilligteVarianten() );

        verbilligte.addAll( waffenspezialisierungen.getVerbilligteVarianten() );

        return verbilligte;
    }

    /** Waehlt alle Eintraege im Panel ab. */
    public void reset() {
        verschiedene.reset();

        talentspezialisierungen.reset();

        waffenspezialisierungen.reset();
    }
}
