/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.FernkampfwaffeWert;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.VariantenListModel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;

class PanelFernkampfwaffe extends JPanel implements Observer {

    private static final String FK = I18n.i18n( "PanelFernkampfwaffe.Fk" );

    private static final String NAME = I18n.i18n( "PanelFernkampfwaffe.Name" );

    private static final String REICHWEITEN =
        I18n.i18n( "PanelFernkampfwaffe.Reichweiten" );

    private static final String TALENT = I18n.i18n( "PanelFernkampfwaffe.Talent" );

    private static final String TP = I18n.i18n( "PanelFernkampfwaffe.Tp" );

    private static final String TP_PLUS =
        I18n.i18n( "PanelFernkampfwaffe.TpPlus" );

    private static final String WAFFE = I18n.i18n( "PanelFernkampfwaffe.Waffe" );

    /** Das Label das den Fernkampfwert anzeigt. */
    private final JLabel fk;

    /** Die Checkbox mit der man die Waffe fuer den Held an-/abwaehlt. */
    private final JCheckBox gewaehlt;

    /** Der Held fuer den mit diesem Panel eine Waffe gewaehlt wird. */
    private Held held;

    /** Das Textfeld fuer den Namen der Waffe. */
    private final JTextField name;

    /** Die Spinner fuer die Reichweiten der Waffe. */
    private final List< JSpinner > reichweiten;

    /** Erlaubt es das Talent, mit dem die Waffe gefuehrt wird, zu waehlen. */
    private final JComboBox talent;

    /** Das Model fuer die Combobox {@link #talent}. */
    private final VariantenListModel talentModel;

    /** Das Textfeld fuer die Trefferpunkte der Waffe. */
    private final JTextField tp;

    /** Die TP+ der Waffe. */
    private final List< JSpinner > tpPlus;

    /** Die Combobox mit der man den Typ der Waffe auswaehlt. */
    private final JComboBox waffe;

    /** Der ActionListener fuer die Combobox {@link #waffe}. */
    private final ActionListener waffeListener;

    /** Die Fernkampfwaffe, die ueber das Panel bearbeitet wird. */
    private FernkampfwaffeWert waffeWert;

    public PanelFernkampfwaffe() {

        fk = new JLabel();

        gewaehlt = new JCheckBox();
        gewaehlt.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                anAbgewaehlt();
            }
        });

        name = new JTextField();
        name.getDocument().addDocumentListener( new DocumentListener(){
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                update();
            }

            public void removeUpdate( final DocumentEvent e ) {
                update();
            }

            private void update() {
                waffeWert.setName( name.getText() );
            }
        } );

        final ChangeListener reichweitenListener = new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                updateReichweiten();
            }
        };

        reichweiten = new ArrayList< JSpinner >( 5 );
        for ( int i = 0 ; i < 5 ; ++i ) {

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 999, 5 );

            spinner.addChangeListener( reichweitenListener );

            reichweiten.add( spinner );
        }

        talentModel = new VariantenListModel();

        talent = new JComboBox( talentModel );
        talent.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                talentGewaehlt();
            }
        });

        tp = new JTextField();
        tp.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                updateTp();
            }

            public void removeUpdate( final DocumentEvent e ) {
                updateTp();
            }
        } );

        final ChangeListener tpPlusListener = new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                updateTpPlus();
            }
        };

        tpPlus = new ArrayList< JSpinner >( 5 );
        for ( int i = 0 ; i < 5 ; ++i ) {

            final JSpinner spinner = Util.erzeugeSpinner( 0, -9, 999, 1 );

            spinner.addChangeListener( tpPlusListener );

            tpPlus.add( spinner );
        }

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        waffeListener = new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                waffeGewaehlt();
            }
        };

        waffe = new JComboBox(
                new Vector< Object >( speicher.getFernkampfwaffen() ));
        waffe.addActionListener( waffeListener );

        waffeWert = new FernkampfwaffeWert(
                (Fernkampfwaffe) waffe.getSelectedItem() );

        aufbauenBenutzeroberflaeche();

        aktualisiereAnzeige();
    }

    /**
     * Wird aufgerufen wenn die Waffe fuer den Helden an-/abgewaehlt wird.
     */
    private void anAbgewaehlt() {

        if ( gewaehlt.isSelected() ) {

            held.einfuegen( waffeWert );

        } else {

            held.entfernen( waffeWert );
        }
    }

    /**
     * Uebernimmt die Werte der {@link #waffeWert} in die Benutzeroberflaeche.
     */
    private void aktualisiereAnzeige() {

        if ( null != held ) {

            fk.setText( String.valueOf( waffeWert.getFernkampfwert( held ) ) );
        }

        name.setText( waffeWert.getName() );

        aktualisiereSpinner( reichweiten, waffeWert.getReichweiten() );

        final Set< Kampftechnik > talente = new TreeSet< Kampftechnik >();
        final Kampftechnik haupttalent = waffeWert.getWaffe().getTalent();
        talente.add( haupttalent );
        talente.addAll( haupttalent.getVerwandteFertigkeiten() );
        talentModel.setItems( talente );

        talent.setSelectedItem( waffeWert.getTalent() );

        tp.setText( waffeWert.getTp().toString() );

        aktualisiereSpinner( tpPlus, waffeWert.getTpPlus() );

        waffe.removeActionListener( waffeListener );
        waffe.setSelectedItem( waffeWert.getWaffe() );
        waffe.addActionListener( waffeListener );
    }

    /**
     * Setzt die Werte in den Spinnern auf die uebergebenen Werte.
     *
     * @param spinner
     *            deren Werte eingestellt werden sollen.
     * @param werte
     *            die die Spinner erhalten sollen.
     */
    private void aktualisiereSpinner(
            final List< JSpinner > spinner,
            final List< Integer > werte ) {

        for ( int i = 0, n = spinner.size() ; i < n ; ++i ) {

            spinner.get( i ).setValue( werte.get( i ) );
        }
    }

    /**
     * Ordnet die Elemente der Benutzeroberflaeche auf dem Panel an.
     */
    private void aufbauenBenutzeroberflaeche() {

        String spalten =
            "P, 4dlu, P, 2dlu, P, 4dlu, P, 2dlu, P, 4dlu, P, 2dlu, 20px";

        String zeilen = "P";

        final FormLayout layout1 = new FormLayout( spalten, zeilen );

        final JPanel zeile2 = new JPanel( layout1 );

        final CellConstraints cc = new CellConstraints();

        zeile2.add( waffe,                  cc.xy(  1, 1 ) );
        zeile2.add( new JLabel( TALENT ), cc.xy(  3, 1 ) );
        zeile2.add( talent,               cc.xy(  5, 1 ) );
        zeile2.add( new JLabel( TP ),     cc.xy(  7, 1 ) );
        zeile2.add( tp,                   cc.xy(  9, 1 ) );
        zeile2.add( new JLabel( FK ),     cc.xy( 11, 1 ) );
        zeile2.add( fk,                   cc.xy( 13, 1 ) );

        final JPanel zeile3 = erzeugePanel( reichweiten );

        final JPanel zeile4 = erzeugePanel( tpPlus );

        spalten = "P, 4dlu, P, 2dlu, P:G";

        zeilen =  "P, 2dlu, P, 2dlu, P, 2dlu, P";

        setLayout( new FormLayout( spalten, zeilen ) );

        add( gewaehlt, cc.xy( 1, 1 ) );

        add( new JLabel( NAME ), cc.xy( 3, 1 ) );
        add( name,               cc.xy( 5, 1 ) );

        add( new JLabel( WAFFE ), cc.xy( 3, 3 ) );
        add( zeile2,            cc.xy( 5, 3 ) );

        add( new JLabel( REICHWEITEN ), cc.xy  ( 3, 5 ) );
        add( zeile3,                    cc.xy  ( 5, 5 ) );

        add( new JLabel( TP_PLUS ), cc.xy( 3, 7 ) );
        add( zeile4,                cc.xy( 5, 7 ) );
    }

    /**
     * Ordnet 5 Spinner hintereinander auf einem Panel an.
     *
     * @param spinner
     *           Die Spinner fuer auf das Panel.
     * @return ein Panel mit den Spinnern.
     */
    private JPanel erzeugePanel( final List< JSpinner > spinner ) {

        final FormLayout layout = new FormLayout(
                "P, 2dlu, P, 2dlu, P, 2dlu, P, 2dlu, P",
                "P" );

        final JPanel panel = new JPanel( layout );

        final CellConstraints cc = new CellConstraints();

        for ( int i = 0 ; i < 5 ; ++i ) {

            final int x = 1 + ( 2 * i );

            panel.add( spinner.get( i ), cc.xy( x, 1 ) );
        }

        return panel;
    }

    /**
     * Bestimmt den Held fuer den man die Waffe waehlt.
     *
     * @param derHeld
     *            fuer den die Waffe gewaehlt wird.
     * @param eineWaffe
     *            die der Held evtl. schon besitzt. Sie kann dann ueber das
     *            Panel manipuliert werden. Es kann {@code null} uebergeben
     *            werden.
     */
    public final void setHeld(
            final Held derHeld,
            final FernkampfwaffeWert eineWaffe ) {

        held = derHeld;

        held.addObserver( this );

        if ( null != eineWaffe ) {

            waffeWert = eineWaffe;

            gewaehlt.setSelected( true );

            aktualisiereAnzeige();

        } else {

            gewaehlt.setSelected( false );

            fk.setText( String.valueOf( waffeWert.getFernkampfwert( held ) ) );
        }
    }

    /**
     * Wird aufgerufen, wenn das Talent gewaehlt wird, mit dem die Waffe
     * gefuehrt wird.
     */
    private void talentGewaehlt() {
        final Kampftechnik technik = (Kampftechnik) talent.getSelectedItem();

        waffeWert.setTalent( technik );

        aktualisiereAnzeige();
    }

    public void update( final Observable o, final Object arg ) {

        fk.setText( String.valueOf( waffeWert.getFernkampfwert( held ) ) );
    }

    /**
     * Wird aufgerufen, wenn die Spinner fuer die Reichweite veraendert werden
     * und aktualisiert die Reichweiten der Waffe.
     */
    private void updateReichweiten() {

        final List< Integer > liste = new ArrayList< Integer >( 5 );

        for ( final JSpinner spinner : reichweiten ) {

            liste.add( (Integer) spinner.getValue() );
        }

        waffeWert.setReichweiten( liste );
    }

    /**
     * Wird aufgerufen, wenn in das Textfeld fuer die Trefferpunkte etwas
     * eingegeben wird, ueberprueft die eingegebene Formel und aktualisiert die
     * TP der Waffe.
     */
    private void updateTp() {

        final String tpString = tp.getText().trim();

        if ( Tp.istTpFormel( tpString ) ) {

            tp.setBackground( name.getBackground() );

            waffeWert.setTp( new Tp( tpString ) );

        } else {

            tp.setBackground( Util.FEHLERFARBE );
        }
    }

    /**
     * Wird aufgerufen, wenn die Spinner fuer die TP+ veraendert werden und
     * aktualisiert die TP+ der Waffe.
     */
    private void updateTpPlus() {

        final List< Integer > liste = new ArrayList< Integer >( 5 );

        for ( final JSpinner spinner : tpPlus ) {

            liste.add( (Integer) spinner.getValue() );
        }

        waffeWert.setTpPlus( liste );
    }

    /**
     * Wird aufgerufen, wenn mit der Combobox {@link #waffe} eine Waffe gewaehlt
     * wird.
     */
    private void waffeGewaehlt() {

        final Fernkampfwaffe fernkampfwaffe =
            (Fernkampfwaffe) waffe.getSelectedItem();

        waffeWert.setWaffe( fernkampfwaffe );

        aktualisiereAnzeige();
    }
}
