/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.awt.Font;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.gui.PanelHeldendaten;
import de.bernhardjung.dsaprogramm.gui.PanelSonderfertigkeiten;
import de.bernhardjung.dsaprogramm.gui.PanelTalente;
import de.bernhardjung.dsaprogramm.gui.PanelZauber;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class PanelSteigern extends JPanel implements Observer {

    /** Der Reiter-Text auf der Registerkarte auf der die AP angezeigt werden.*/
    private static final String AP = I18n.i18n( "PanelSteigern.Ap" );

    /** Der Text fuer den InternalFrame in dem die AP angezeigt werden. */
    private static final String ABENTEUERPUNKTE =
        I18n.i18n( "PanelSteigern.Abenteuerpunkte" );

    private static final String AP_ZUR_VERFUEGUNG =
        I18n.i18n( "PanelSteigern.ApZurVerfuegung" );

    private static final String AUSRUESTUNG =
        I18n.i18n( "PanelSteigern.Ausruestung" );

    private static final String BASISWERTE =
        I18n.i18n( "PanelSteigern.Basiswerte" );

    private static final String EIGENSCHAFTEN  =
        I18n.i18n( "PanelSteigern.Eigenschaften" );

    private static final String HELDENDATEN  =
        I18n.i18n( "PanelSteigern.Heldendaten" );

    private static final String SONDERFERTIGKEITEN =
        I18n.i18n( "PanelSteigern.Sonderfertigkeiten" );

    private static final String TALENTE =
        I18n.i18n( "PanelSteigern.Talente" );

    private static final String VOR_NACHTEILE =
        I18n.i18n( "PanelSteigern.VorNachteile" );

    private static final String ZAUBER =
        I18n.i18n( "PanelSteigern.Zauber" );

    private final PanelAp ap;

    private final JLabel apGuthaben;

    private final PanelAusruestung ausruestung;

    private final PanelEigenschaftenSteigern eigenschaften;

    private final PanelBasiswerteSteigern basiswerte;

    /** Der Held der aktuell gesteigert wird. */
    private Held held;

    private final PanelHeldendaten heldendaten;

    private final PanelSonderfertigkeiten sonderfertigkeiten;

    private final PanelTalente talente;

    private final PanelVorNachteileSteigern vorNachteile;

    private final PanelZauber zauber;

    public PanelSteigern() {

        ap = new PanelAp();

        apGuthaben = new JLabel();
        apGuthaben.setHorizontalAlignment( JLabel.RIGHT );
        final Font schrift = getFont();
        final int schriftgroesse = schrift.getSize();
        final float neueSchriftgroesse = schriftgroesse * 1.5F;
        final Font grosseSchrift =
            schrift.deriveFont( Font.BOLD, neueSchriftgroesse );
        apGuthaben.setFont( grosseSchrift );

        ausruestung = new PanelAusruestung();

        basiswerte = new PanelBasiswerteSteigern();

        eigenschaften = new PanelEigenschaftenSteigern();

        heldendaten = new PanelHeldendaten();

        sonderfertigkeiten = new PanelSonderfertigkeiten( true );

        talente = new PanelTalente( true );

        vorNachteile = new PanelVorNachteileSteigern();

        zauber = new PanelZauber( true );

        aufbauenBenutzeroberflaeche();
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JPanel apPanel = new JPanel(
                new FormLayout( "7dlu, F:P:G, 7dlu", "7dlu, P, 7dlu ") );
        apPanel.add( apGuthaben, new CellConstraints( 2, 2 ) );

        final JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab(
                HELDENDATEN,
                erzeugeFrame( HELDENDATEN, heldendaten ) );

        tabbedPane.addTab(
                AP,
                erzeugeFrame( ABENTEUERPUNKTE, ap ) );

        tabbedPane.addTab(
                EIGENSCHAFTEN,
                erzeugeFrame( EIGENSCHAFTEN, eigenschaften ) );

        tabbedPane.addTab(
                BASISWERTE,
                erzeugeFrame( BASISWERTE, basiswerte ) );

        tabbedPane.addTab( TALENTE, talente );

        tabbedPane.addTab( VOR_NACHTEILE, vorNachteile );

        tabbedPane.addTab( SONDERFERTIGKEITEN, sonderfertigkeiten );

        tabbedPane.addTab( ZAUBER, zauber );

        tabbedPane.addTab( AUSRUESTUNG, ausruestung );

        setLayout( new BorderLayout() );

        add( apPanel, BorderLayout.PAGE_START );

        add( tabbedPane, BorderLayout.CENTER );
    }

    /**
     * Erzeugt einen SimpleInternalFrame.
     *
     * @param titel
     *            des Frames.
     * @param inhalt
     *            des Frames.
     * @return einen SimpleInternalFrame.
     */
    private SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JPanel inhalt ) {

        final SimpleInternalFrame frame =
            new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( inhalt );

        return frame;

    }

    /** @return den Held der aktuell gesteigert wird. */
    public final Held getHeld() {
        return held;
    }

    /**
     * @param derHeld der im Panel gesteigert wird.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        ap.setHeld( held );

        ausruestung.setHeld( held );

        basiswerte.setHeld( held );

        eigenschaften.setHeld( held );

        heldendaten.setHeld( held );

        sonderfertigkeiten.setHeld( held );

        talente.setHeld( held );

        vorNachteile.setHeld( held );

        zauber.setHeld( held );

        update( held, null );
    }

    /**
     * @param o der Held, der vom Panel beobachtet wird.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        apGuthaben.setText( String.format( AP_ZUR_VERFUEGUNG,
                                           ((Held) o).getApGuthaben() ) );
    }
}
