/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.i18n;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

/** 
 * Hilfsdatei fuer die Internationalisierung und Lokalisierung der Anwendung.
 * Die Methode {@link #i18n(String)} kann verwendet werden um die 
 * lokalisierten Texte abzurufen. 
 */
public class I18n {
    
    /** Das ResourceBundle in dem die Lokalisierten Texte gespeichert sind. */
    private static ResourceBundle RESOURCE_BUNDLE = ResourceBundle.getBundle(
            "de.bernhardjung.dsaprogramm.i18n.Lokalisierung" );
    
    /**
     * Prueft ob es fuer einen Schluessel einen Lokalisierungstext gibt.
     * 
     * @param schluessel zu dem der Text gesucht ist.
     * @return {@code true}, wenn ein Text zum Schlussel existiert, sonst
     *      {@code false}.
     */
    public static final boolean hatLokalisierungstext( 
            final String schluessel ) {
        
        try {
            
            RESOURCE_BUNDLE.getString( schluessel );
            
            return true;
            
        } catch ( MissingResourceException e ) {
            
            return false;
            
        }
    }
    
    /**
     * Liefert zu einem gegebenen Schluessel einen lokalisierten Text.
     * 
     * @param schluessel zu dem der lokalisierte Text gesucht ist.
     * @return den zum {@code schluessel} gehoerenden Text. Wenn kein zum 
     *      Schluessel gehoerender Text gefunden wird, wird {@code null} 
     *      geliefert.
     */
    public static final String i18n( final String schluessel ) {
        
        final String text = RESOURCE_BUNDLE.getString( schluessel );
        
        if ( null != text ) {
            
            return text;
            
        } else {
        
            System.out.printf( "Lokaliserungstext fuer Schluessel %1$s " +
                    "nicht gefunden.\n", schluessel );
            
            return null;
            
        }
    }
    
}
