/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/** Berechnet die Kosten fuer die Sonderfertigkeit Ortskenntnis. */
class KostenOrtskenntnis extends KostenAbstrakt {

    /** Die AP-Kosten fuer die erste Ortskenntnis. */
    private static final int AP_KOSTEN_ERSTE = 150;

    /** Die AP-Kosten fuer die 2. und weiteren Ortskenntnisse. */
    private static final int AP_KOSTEN_WEITERE = 100;


    /** @return Elfische Weltsicht. */
    public Collection<Object> getAbhaengigkeiten() {

        final List< Object > abhaengigkeiten = new ArrayList< Object >();

        abhaengigkeiten.add( VorNachteil.ELFISCHE_WELTSICHT );

        return abhaengigkeiten;
    }

    public int getApKosten(Sonderfertigkeit sonderfertigkeit) {
        return AP_KOSTEN_ERSTE;
    }

    public int getApKosten(final SonderfertigkeitWert wert) {
        return AP_KOSTEN_ERSTE;
    }

    public int getApKosten( final Held held, final SonderfertigkeitWert wert) {

        if ( held.hatVorgegeben( wert ) ) {
            // vorgegebene Werte kosten nichts.
            return 0;
        }

        final float faktor;

        if ( held.hatVerbilligt( wert ) ) {

            faktor = 0.5F;

        } else if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
                && ( ! held.hatLeitsonderfertigkeit( wert ) ) ) {

            faktor = 1.5F;

        } else {

            faktor = 1.0F;

        }

        if ( held.istErsteOrtskenntnis( wert ) ) {

            return Math.round( faktor * AP_KOSTEN_ERSTE );

        } else {

            return Math.round( faktor * AP_KOSTEN_WEITERE );

        }
    }

}
