/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Fuer Talente, die einen bestimmten Vor-/Nachteil voraussetzen, z.B. die
 * Gaben.
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungVorteil extends Voraussetzung {

    /** Erklaert dem Benutzer, dass der Vorteil benoetigt wird. */
    private static final String BENOETIGT =
        I18n.i18n( "VoraussetzungVorteil" );

    private final VorNachteil vorteil;

    /**
     * @param derVorteil der vorausgesetzt wird.
     */
    VoraussetzungVorteil( final VorNachteil derVorteil ) {

        vorteil = derVorteil;
    }

    /** @return den Vorteil. */
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) vorteil );
    }

    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final Talent talent ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( ! held.hat( vorteil ) ) {

            beschreibung.einfuegen( String.format( BENOETIGT, vorteil ) );
        }
        return beschreibung;
    }

}
