/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.util;

import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Eine Exception die geworfen wird wenn von einer ueberladenen Methode
 * eine Version aufgerufen wird, bei der ein benoetigtes Argument fehlt.
 * <dl>
 * <dt>Beispiel:</dt>
 * <dd>
 *  Wenn eine Sonderfertigkeit Varianten besitzt muss der Konstruktor {@link
 *  SonderfertigkeitWert#SonderfertigkeitWert(Sonderfertigkeit, Object)}
 *  aufgerufen werden weil man die Variante des Wertes angeben muss.
 *  Wird nur der Konstruktor
 *  {@link SonderfertigkeitWert#SonderfertigkeitWert(Sonderfertigkeit)}
 *  aufgerufen wird eine FehlendesArgumentException geworfen.
 * </dd>
 * </dl>
 *
 * @author Bernhard Jung
 */
public class FehlendesArgumentException extends RuntimeException {

    /**
     * Erzeugt eine neue Exception.
     *
     * @param fehlermeldung fuer die Exception.
     */
    public FehlendesArgumentException( final String fehlermeldung ) {
        super( fehlermeldung );
    }
}
