/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Elfen und Magier muessen fuer Gutes Gedaechtnis mehr bezahlen.
 *
 * @author Bernhard Jung
 *
 */
class KostenGutesGedaechtnis implements Kosten {

    /** Die Kosten die Elfen und Magier zu entrichten haben. */
    static final int GP_KOSTEN_ELFEN_MAGIER = 12;

    /** Die Kosten die alle anderen bezahlen muessen. */
    static final int GP_KOSTEN_ANDERE = 7;

    /** @return 0 */
    public int getAbbaukosten(final Held held, final VorNachteilWert wert) {
        return 0;
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    public int getGpKosten() {
        return GP_KOSTEN_ELFEN_MAGIER;
    }

    public int getGpKosten( final VorNachteilWert wert ) {
        return getGpKosten();
    }

    public int getGpKosten( final Held held, final VorNachteilWert wert ) {

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        final Rasse elfen = speicher.getRasse( "Rasse.Elfen" );

        final Profession magier = speicher.getProfession("Profession.Magier");

        if ( held.getRasse().equals( elfen )
          || held.getProfession().equals( magier ) ) {

            return GP_KOSTEN_ELFEN_MAGIER;

        } else {

            return GP_KOSTEN_ANDERE;

        }
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar() {
        return false;
    }
}
